import component from 'kompo';
var construct = component.construct,
    react = component.react,
    mount = component.mount,
    getState = component.getState;


import { create, createFragment, addClasses, throttle, delegate, empty, isObject } from 'kompo-util';

import listitem from './listitem';

export default construct('div', function (_ref) {
    var _this = this;

    var defaultClass = _ref.defaultClass,
        classes = _ref.classes,
        containerClass = _ref.containerClass,
        inputClass = _ref.inputClass,
        listClass = _ref.listClass,
        noResultsClass = _ref.noResultsClass,
        emptyClass = _ref.emptyClass,
        throttleDelay = _ref.throttleDelay,
        blurDelay = _ref.blurDelay,
        noResultsInputRender = _ref.noResultsInputRender,
        noResultsText = _ref.noResultsText,
        showNoResults = _ref.showNoResults,
        placeholder = _ref.placeholder,
        actionClass = _ref.actionClass,
        actionText = _ref.actionText,
        actionCallback = _ref.actionCallback,
        name = _ref.name,
        required = _ref.required,
        disabled = _ref.disabled;

    if (!name) {
        throw new Error('A name is required');
    }

    this.classList.add(defaultClass);
    this.classList.add(emptyClass);
    addClasses(this, classes);

    var props = this.kompo.props,
        container = create('div', { 'class': containerClass }),
        input = create('input', {
        'class': inputClass,
        placeholder: placeholder
    }),
        list = create('ul', { 'class': listClass }),
        noResults = create('li', { 'class': noResultsClass });

    this.kompo.props.__input__ = input;

    if (required) {
        this.setAttribute('required', required);
        input.setAttribute('required', required);
    }

    if (disabled) {
        this.setAttribute('disabled', disabled);
        input.setAttribute('disabled', disabled);
    }

    noResults.textContent = noResultsText;

    /**
     * Structure
     */
    container.appendChild(input);
    this.appendChild(container);
    this.appendChild(list);

    /**
     * Events & Reactions
     */
    if (actionCallback) {
        var action = create('a', { 'class': actionClass, href: '#' });
        action.textContent = actionText;
        container.appendChild(action);

        action.addEventListener('click', function (e) {
            e.preventDefault();
            if (props.selected && props.selected.hasOwnProperty('kompo')) {
                actionCallback(props.selected, input.value);
            }
        });
    }

    input.addEventListener('keyup', throttle(function (e) {
        if (!noResultsInputRender && input.value == '') {
            emptyList(_this, list, emptyClass);
            _this.classList.remove(noResultsClass);
        } else {
            reactFn(true)(getState(_this));
        }

        _this.setAttribute('value', input.value);

        props.selected = list.children[0];
    }, throttleDelay));

    input.addEventListener('blur', function (e) {
        setTimeout(function () {
            emptyList(_this, list, emptyClass);
        }, blurDelay);
    });

    delegate(list, 'li:not(.' + noResultsClass + ')', 'click', function (e) {
        e.preventDefault();
        props.selected = e.target;
        input.value = props.selected.textContent;
        _this.setAttribute('value', input.value);
        emptyList(_this, list, emptyClass);
    });

    var reactFn = function reactFn(usedInput) {
        return function (state) {
            if (isObject(state) && !Array.isArray(state)) {
                if (!usedInput) {
                    var v = state.values[name] || '';
                    input.value = v;
                    _this.setAttribute('value', v);
                }
                state = state.data;
            }

            if (!Array.isArray(state)) return;

            var frag = createFragment(),
                filter = props.filter;

            var _loop = function _loop(i, l) {
                var value = filter ? filter(state[i], input.value) : state[i];

                if (value === false) return 'continue';

                var li = listitem();
                li.textContent = value;
                mount(_this, li, function () {
                    return state[i];
                });
                frag.appendChild(li);
            };

            for (var i = 0, l = state.length; i < l; ++i) {
                var _ret = _loop(i, l);

                if (_ret === 'continue') continue;
            }

            empty(list);

            switch (true) {
                case showNoResults && frag.children.length === 0:
                    _this.classList.add(noResultsClass);
                    list.appendChild(noResults);
                case frag.children.length === 0 || props.skipInitialListRender:
                    _this.classList.add(emptyClass);
                    props.skipInitialListRender = false;
                    break;
                case usedInput:
                    _this.classList.remove(noResultsClass);
                    _this.classList.remove(emptyClass);
                    list.appendChild(frag);
                    break;
            }
        };
    };

    react(this, reactFn(false));
}, {
    defaultClass: 'o-Autocomplete',
    classes: [],
    containerClass: 'o-Autocomplete-container',
    inputClass: 'o-Autocomplete-input',
    listClass: 'o-Autocomplete-list',
    emptyClass: 'o-Autocomplete--isEmpty',
    noResultsClass: 'o-Autocomplete--noResults',
    filter: undefined,
    throttleDelay: 200,
    blurDelay: 400,
    skipInitialListRender: true,
    noResultsInputRender: false,
    noResultsText: 'No results found',
    showNoResults: true,
    placeholder: 'Start typing and select...',
    selected: undefined,
    actionClass: 'o-Autocomplete-action',
    actionText: 'action',
    actionCallback: undefined
});

function emptyList(ac, list, clss) {
    empty(list);
    ac.classList.add(clss);
}

export function reset(autocomplete) {
    var props = autocomplete.kompo.props;

    props.__input__.value = '';
    autocomplete.setAttribute('value', '');

    if ("createEvent" in document) {
        var evt = document.createEvent("HTMLEvents");
        evt.initEvent("keyup", false, true);
        props.__input__.dispatchEvent(evt);
    } else {
        props.__input__.fireEvent("keyup");
    }
}