import component from 'kompo';
import { create, isObject, throttle } from 'kompo-util';

import getDynamicWidth from '../utils/getDynamicWidth';

import '../../css/multiPanel.css';

var getProps = component.getProps,
    mount = component.mount;


var multiPanel = component.construct('div', function (_ref) {
    var _this = this,
        _classList;

    var classNames = _ref.classNames,
        overlay = _ref.overlay,
        unit = _ref.unit,
        transitionDuration = _ref.transitionDuration,
        easingFunction = _ref.easingFunction,
        children = _ref.children;

    var wrapper = create('div'),
        props = getProps(this);

    window.addEventListener('resize', throttle(function (e) {
        var index = children.indexOf(props.activePanel);

        // No component is routed abort all work
        if (index < 0) return;

        // Reset totalWidth
        var changedChildren = [];
        props.totalWidth = 0;

        for (var i = 0, l = children.length; i < l; ++i) {
            var child = children[i],
                childProps = getProps(child),
                basis = childProps.basis;

            if (isObject(basis)) {
                var changedTo = getDynamicWidth(basis);
                if (changedTo === childProps.activeWidth) {
                    props.totalWidth = props.totalWidth + childProps.activeWidth;
                } else {
                    props.totalWidth = props.totalWidth + changedTo;
                    childProps.activeWidth = changedTo;
                    changedChildren.push(true);
                }
            } else {
                props.totalWidth = props.totalWidth + childProps.basis;
            }
        }

        // If no children have changed don't change anything
        if (changedChildren.length === 0) return;

        props.totalWidthPercentage = 100 / props.totalWidth * 100;

        for (var _i = 0, _l = children.length; _i < _l; ++_i) {
            var _child = children[_i],
                _childProps = getProps(_child),
                _basis = _childProps.basis,
                percentage = isObject(_basis) ? _childProps.activeWidth / 100 * props.totalWidthPercentage : _basis / 100 * props.totalWidthPercentage,
                pu = percentage + unit;

            _child.style.width = pu;
        }

        // Use double request animation frames to make sure
        // the panels do not animate on resize
        wrapper.style.transition = 'none';
        requestAnimationFrame(function () {
            slideTo(_this, children, index);
            requestAnimationFrame(function () {
                wrapper.style.transition = 'transform ' + transitionDuration + 'ms';
            });
        });
    }, 200));

    classNames.push('o-MultiPanel');
    if (overlay) classNames.push('o-MultiPanel--withOverlay');
    (_classList = this.classList).add.apply(_classList, classNames);

    for (var i = 0, l = children.length; i < l; ++i) {
        var child = children[i],
            childProps = getProps(child);

        childProps.index = i;
        childProps.slideTo = function (index) {
            return slideTo(_this, children, index);
        };

        if (isObject(childProps.basis)) {
            var dw = getDynamicWidth(childProps.basis);
            props.totalWidth = props.totalWidth + dw;
            childProps.activeWidth = dw;
        } else {
            props.totalWidth = props.totalWidth + childProps.basis;
        }

        childProps.multiPanelProps = props;
        wrapper.appendChild(child);
    }

    props.totalWidthPercentage = 100 / props.totalWidth * 100;

    wrapper.style.width = props.totalWidth + unit;
    requestAnimationFrame(function () {
        wrapper.style.transition = 'transform ' + transitionDuration + 'ms ' + easingFunction;
    });

    props.wrapper = wrapper;

    // Pass on selector
    mount(this, children, this.kompo.selector);
    this.appendChild(wrapper);
}, {
    classNames: [],
    unit: '%',
    transitionDuration: 500,
    totalWidth: 0,
    wrapper: undefined,
    overlay: true,
    easingFunction: 'ease',
    children: []
});

export default multiPanel;

export function slideTo(multiPanel, panels, index) {
    var initial = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

    var translateTo = 0,
        totalPercentage = 0;

    var props = getProps(multiPanel);

    for (var i = 0, l = panels.length; i < l; ++i) {
        var panel = panels[i];

        if (i < index) {
            translateTo = translateTo + getWidth(panel);
        }

        if (!initial && !panel.classList.contains('o-MultiPanel-panel--withOverlayTransition')) {
            panel.classList.add('o-MultiPanel-panel--withOverlayTransition');
        }

        if (i === index) {
            panel.classList.add('o-MultiPanel-panel--selected');
            props.activePanel = panel;
        } else {
            panel.classList.remove('o-MultiPanel-panel--selected');
        }

        totalPercentage = totalPercentage + getWidth(panel);
    }

    if (index == panels.length - 1) {
        var lastPanel = panels[panels.length - 1],
            basis = lastPanel.kompo.props.basis,
            lastBasis = isObject(basis) ? getDynamicWidth(basis) : basis;

        translateTo = translateTo - (100 - lastBasis) / 100 * props.totalWidthPercentage;
    }

    var unit = props.unit;
    props.wrapper.style.width = props.totalWidth + unit;
    props.wrapper.style.transform = 'translateX(-' + translateTo + unit + ')';
}

export function slide(component, router, element) {
    var multiPanelProps = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

    var mp = void 0;

    return {
        do: function _do() {
            var c = router.get(component, true);

            var panels = [],
                routes = c.kompo.siblingRoutes;

            var _loop = function _loop(i, l) {
                var route = routes[i],
                    co = route.component;

                // TODO This does not work as expected
                // when the component of this panel
                // contains child routes. Only goes back to
                // original path, not the child routes
                co.kompo.props.slideToUrl = function () {
                    router.goTo(route.path);
                    return route.path;
                };

                panels.push(co);
            };

            for (var i = 0, l = routes.length; i < l; ++i) {
                _loop(i, l);
            }

            var routed = component.kompo.routed;

            if (c === routed) return;

            var index = panels.indexOf(c),
                el = element ? element : component;

            var initial = false;
            if (!routed) {
                multiPanelProps.children = panels;
                mp = multiPanel(multiPanelProps);
                mount(component, mp);
                el.appendChild(mp);
                initial = true;
            }

            slideTo(mp, panels, index, initial);
            component.kompo.routed = c;
        }
    };
}

function getWidth(el) {
    return parseFloat(el.style.width);
}