import component, { state } from 'kompo';
var construct = component.construct,
    react = component.react,
    mount = component.mount,
    unmountAll = component.unmountAll;
var dispatch = state.dispatch;


import { addClasses, create, empty, delegate, throttle } from 'kompo-util';

import arrayMove from '../utils/arrayMove';
import orderByArray from '../utils/orderByArray';

var item = construct('li', function (_ref) {
    var _this = this;

    var defaultClass = _ref.defaultClass,
        classes = _ref.classes,
        name = _ref.name,
        alias = _ref.alias,
        disabled = _ref.disabled,
        disabledClass = _ref.disabledClass,
        checkClass = _ref.checkClass,
        wrapperClass = _ref.wrapperClass,
        placeholder = _ref.placeholder;

    this.classList.add(defaultClass);
    addClasses(this, classes);

    var check = create('a', { 'class': checkClass, href: '#activateOrDisable' }),
        wrapper = create('div', { 'class': wrapperClass }),
        nameEl = create('span'),
        aliasEl = create('input', { placeholder: placeholder });

    nameEl.textContent = name;
    if (alias) aliasEl.value = alias;

    wrapper.appendChild(nameEl);
    wrapper.appendChild(aliasEl);
    this.appendChild(check);
    this.appendChild(wrapper);

    if (disabled) {
        this.classList.add(disabledClass);
    }

    /**
     * Events
     */
    aliasEl.addEventListener('keyup', throttle(function (e) {
        dispatch(_this, function (state) {
            state.mapping[name] = aliasEl.value;
        });
    }, 100));
}, {
    defaultClass: 'o-OrderList-item',
    classes: [],
    name: 'N/A',
    alias: '',
    index: undefined,
    disabled: false,
    disabledClass: 'o-OrderList-item--disabled',
    checkClass: 'o-OrderList-item-check',
    wrapperClass: 'o-OrderList-item-wrapper',
    placeholder: 'Provide an alias for this column...'
});

export default construct('div', function (_ref2) {
    var _this3 = this;

    var defaultClass = _ref2.defaultClass,
        classes = _ref2.classes,
        upText = _ref2.upText,
        downText = _ref2.downText,
        available = _ref2.available,
        disabledClass = _ref2.disabledClass,
        wrapperClass = _ref2.wrapperClass,
        upClass = _ref2.upClass,
        downClass = _ref2.downClass,
        listClass = _ref2.listClass,
        itemClass = _ref2.itemClass,
        itemWrapperClass = _ref2.itemWrapperClass,
        itemCheckClass = _ref2.itemCheckClass,
        itemPlaceholder = _ref2.itemPlaceholder,
        itemDisabledClass = _ref2.itemDisabledClass;

    this.classList.add(defaultClass);
    addClasses(this, classes);

    var orderList = this,
        props = this.kompo.props,
        wrapper = create('div', { 'class': wrapperClass }),
        up = create('a', { 'class': upClass, href: '#up' }),
        down = create('a', { 'class': downClass, href: '#down' }),
        list = create('ol', { 'class': listClass });

    up.textContent = upText;
    down.textContent = downText;
    wrapper.appendChild(up);
    wrapper.appendChild(down);
    this.appendChild(wrapper);
    this.appendChild(list);

    /**
     * Eventhandlers
     */
    var reorderFn = function reorderFn(im) {
        return function (e) {
            e.preventDefault();

            if (!props.selectedItem) return;

            var name = props.selectedItem.kompo.props.name,
                ai = available.indexOf(name),
                aim = ai + im;

            arrayMove(available, ai, aim >= available.length ? 0 : aim);
            dispatch(orderList, function (state) {
                orderByArray(state.want, available);
            });
        };
    };
    up.addEventListener('click', reorderFn(-1));
    down.addEventListener('click', reorderFn(1));

    delegate(this, 'li', 'click', function (e) {
        var _this2 = this;

        e.preventDefault();

        if (e.target.tagName === 'A') {
            dispatch(orderList, function (state) {
                toggleItem(_this2, state);
            });
        } else if (this.kompo.props.disabled === false) {
            setSelected(orderList, this);
        }
    });

    var ordered = false;

    /**
     * Reactions
     */
    var reactFn = function reactFn(state) {
        unmountAll(_this3);
        empty(list);

        var want = state.want || [],
            mapping = state.mapping || {},
            ics = [],
            selectedItem = props.selectedItem,
            selectedName = selectedItem ? selectedItem.kompo.props.name : undefined;

        if (!ordered) {
            orderByArray(available, want);
            ordered = true;
        }

        for (var i = 0, l = available.length; i < l; ++i) {
            var name = available[i],
                hasMapping = mapping.hasOwnProperty(name),
                ic = item({
                name: name,
                alias: hasMapping ? mapping[name] : undefined,
                index: i,
                defaultClass: itemClass,
                wrapperClass: itemWrapperClass,
                checkClass: itemCheckClass,
                placeholder: itemPlaceholder,
                disabledClass: itemDisabledClass
            });

            if (want.indexOf(name) === -1) {
                ic.classList.add(disabledClass);
                ic.kompo.props.disabled = true;
            } else {
                ic.classList.remove(disabledClass);
                ic.kompo.props.disabled = false;
            }

            if (selectedItem && selectedName === name) {
                setSelected(orderList, ic);
            }

            ics.push(ic);
            list.appendChild(ic);
        }

        mount(_this3, ics, _this3.kompo.selector);
        _this3.appendChild(list);
    };
    react(this, reactFn);
}, {
    defaultClass: 'o-OrderList',
    classes: [],
    upText: 'Up',
    downText: 'Down',
    available: [],
    selectedItem: undefined,
    selectedClass: 'o-OrderList-item--selected',
    disabledClass: 'o-OrderList-item--disabled',
    wrapperClass: 'o-OrderList-wrapper',
    upClass: 'o-OrderList-up',
    downClass: 'o-OrderList-down',
    listClass: 'o-OrderList-list',
    itemClass: 'o-OrderList-item',
    itemDisabledClass: 'o-OrderList-item--disabled',
    itemWrapperClass: 'o-OrderList-item-wrapper',
    itemCheckClass: 'o-OrderList-item-check',
    itemPlaceholder: 'Provide an alias for this column...'
});

function setSelected(orderList, item) {
    var props = orderList.kompo.props,
        selectedItem = props.selectedItem;

    if (selectedItem === item) return;

    if (selectedItem) {
        selectedItem.classList.remove(props.selectedClass);
    }

    item.classList.add(props.selectedClass);
    props.selectedItem = item;
}

function toggleItem(item, state) {
    var props = item.kompo.props,
        name = props.name,
        disabledClass = props.disabledClass;

    if (props.disabled) {
        item.classList.remove(disabledClass);
        props.disabled = false;
        state.want.splice(state.want.indexOf(name), 0, name);
    } else {
        item.classList.add(disabledClass);
        props.disabled = true;
        state.want.splice(state.want.indexOf(name), 1);
    }
}