var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import component, { state } from 'kompo';
var dispatch = state.dispatch,
    ignoreUpdate = state.ignoreUpdate,
    resetIgnoreUpdate = state.resetIgnoreUpdate;

import { empty, isFunction, addClasses, throttle, isObject } from 'kompo-util';

import tableRow from './tableRow';
import tableHead from './tableHead';

export default component.construct('table', function (_ref) {
    var _this = this;

    var appendRow = _ref.appendRow,
        on = _ref.on,
        classes = _ref.classes,
        oddRowClass = _ref.oddRowClass,
        evenRowClass = _ref.evenRowClass,
        selectedClass = _ref.selectedClass,
        appendHead = _ref.appendHead,
        headRowClass = _ref.headRowClass,
        emptyNotice = _ref.emptyNotice,
        emptyClass = _ref.emptyClass,
        scrollableElement = _ref.scrollableElement,
        topSpacer = _ref.topSpacer,
        bottomSpacer = _ref.bottomSpacer,
        rowHeight = _ref.rowHeight,
        blockSize = _ref.blockSize,
        scrollThrottle = _ref.scrollThrottle,
        uniqueKey = _ref.uniqueKey;

    // Prevent memory sharing of selected between instances of infiniteTables
    this.kompo.props.selected = _extends({}, this.kompo.props.selected);

    var head = document.createElement('thead'),
        body = document.createElement('tbody'),
        props = component.getProps(this),
        frag = document.createDocumentFragment();

    if (isFunction(on)) on(this);

    addClasses(this, classes);

    var wasEmpty = false;

    scrollableElement = isFunction(scrollableElement) ? scrollableElement() : scrollableElement;

    if (scrollableElement) {
        scrollableElement.style.overflowAnchor = 'none'; // Prevent weird Chrome 56+ behavior
        scrollableElement.addEventListener('scroll', throttle(function (e) {
            dispatch(_this, function (state) {
                var dataLength = state.data.length,
                    scrollPosition = scrollableElement.scrollTop,

                // TODO clientHeight could be troublesome
                reqPerBody = (Math.ceil(scrollableElement.clientHeight / rowHeight / blockSize) + 2) * blockSize,
                    visibleStart = Math.floor(scrollPosition / rowHeight),
                    renderStart = Math.max((Math.floor(visibleStart / blockSize) - 1) * blockSize, 0),
                    visibleEnd = Math.min(renderStart + reqPerBody, dataLength);

                if (state.offset !== renderStart || state.limit !== visibleEnd - renderStart) {
                    state.offset = renderStart;
                    state.limit = visibleEnd - renderStart;
                }
            });
        }, scrollThrottle));
    }

    var reactFn = function reactFn(state) {
        var data = state.data;
        var offset = state.offset,
            limit = state.limit,
            minimize = state.minimize,
            rowFilter = props.rowFilter,
            columnFilter = props.columnFilter;

        if (!Array.isArray(data)) return;
        if (!Array.isArray(data)) {
            if (isObject(data)) {
                data = Object.values(data);
            } else {
                return;
            }
        }

        // Empty out before append
        empty(head);
        component.unmountAll(_this);

        // When empty...
        if (data.length == 0) {
            // ... render empty notice
            var tr = document.createElement('tr'),
                td = document.createElement('td');

            td.textContent = emptyNotice;
            tr.appendChild(td);
            head.appendChild(tr);
            _this.classList.add(emptyClass);
            wasEmpty = true;
            empty(body);
            return;
        } else if (wasEmpty) {
            // Remove empty notice
            _this.classList.remove(emptyClass);
            wasEmpty = false;
        }

        var hasRowFilter = isFunction(rowFilter),
            hasColumnFilter = isFunction(columnFilter);

        appendHead(_this, head, {
            defaultClass: headRowClass,
            minimizeWhitelist: props.minimizeWhitelist,
            hasColumnFilter: hasColumnFilter,
            columnFilter: columnFilter
        });

        var dataLength = data.length;
        var os = offset && offset <= dataLength ? offset : 0;
        var l = typeof limit !== 'undefined' && os + limit < dataLength ? os + limit : dataLength;

        // Set spacer heights
        bottomSpacer.style.height = rowHeight * (dataLength - (offset + limit)) + 'px';
        topSpacer.style.height = rowHeight * offset + 'px';

        for (os; os < l; ++os) {
            if (hasRowFilter && !rowFilter(data[os])) continue;
            appendRow(_this, frag, os, {
                defaultClass: os % 2 == 0 ? evenRowClass : oddRowClass,
                selectedClass: selectedClass,
                hasColumnFilter: hasColumnFilter,
                columnFilter: columnFilter,
                selected: props.selected,
                uniqueKey: uniqueKey
            });
        }

        empty(body);
        body.appendChild(frag);
    };

    component.react(this, reactFn);

    this.appendChild(head);
    this.appendChild(body);
}, {
    classes: [],
    oddRowClass: '',
    evenRowClass: '',
    headRowClass: '',
    minimizeWhitelist: undefined,
    emptyNotice: 'No data found',
    emptyClass: 'empty',
    appendRow: function appendRow(table, frag, key, props) {
        var tr = tableRow(props);
        component.mount(table, tr, function (s) {
            var ns = table.kompo.selector ? table.kompo.selector(s).data[key] : s.data[key];

            if (ns && props.selected && ns.hasOwnProperty(props.uniqueKey)) {
                // Prevents selected from being picked up by the state store and user data
                Object.defineProperty(ns, '__kompo_selected__', {
                    writable: true,
                    value: props.selected.hasOwnProperty(ns[props.uniqueKey])
                });
            }

            return ns;
        });
        frag.appendChild(tr);
    },
    appendHead: function appendHead(table, head, props) {
        var tr = tableHead(props);
        component.mount(table, tr, function (s) {
            return table.kompo.selector ? table.kompo.selector(s).data[0] : s.data[0];
        });
        head.appendChild(tr);
    },

    scrollableElement: undefined,
    topSpacer: undefined,
    bottomSpacer: undefined,
    rowHeight: 20,
    blockSize: 5,
    scrollThrottle: 10,
    uniqueKey: 'id',
    selected: {}
});

export function resetSpacers(infiniteTable) {
    var props = infiniteTable.kompo.props;
    props.topSpacer.style.height = 0;
    props.bottomSpacer.style.height = 0;
}

export function multiSelect(table, e) {
    var target = e.target.parentNode,
        tableProps = table.kompo.props,
        key = keySelected(tableProps, target);

    // Remove if already in selected
    if (tableProps.selected.hasOwnProperty(key)) {
        target.classList.remove(tableProps.selectedClass);
        delete tableProps.selected[key];
        return;
    }

    tableProps.selected[key] = target;
    target.classList.add(tableProps.selectedClass);
}

function keySelected(tableProps, row) {
    var s = component.getState(row);
    return s[tableProps.uniqueKey];
}